package org.greenstone.gsdl3.gs3build.util;

import java.util.ArrayList;
import java.util.List;

import org.greenstone.gsdl3.gs3build.doctypes.DocumentID;

public class FileTrie
{	String   stem;
	List     childTries;
	List     childDocs;
	FileTrie parent;

	public FileTrie(String stem)
	{	this.childTries = new ArrayList();
		this.childDocs  = new ArrayList();
		this.stem = stem;
		this.parent = null;
	}

	public void setParent(FileTrie trie)
	{	this.parent = trie;
	}

	public FileTrie getParent()
	{	return this.parent;
	}

	public void addChild(FileTrie trie)
	{	trie.setParent(this);
		this.childTries.add(trie);
	}

	public void replaceChild(FileTrie oldChild, FileTrie newChild)
	{	for (int i = 0; i < this.childTries.size(); i ++)
		{	FileTrie thisChild = (FileTrie) this.childTries.get(i);
			if (thisChild == oldChild)
			{	this.childTries.set(i, newChild);
				return;
			}
		}
	}

	public void addDocument(DocumentID documentID)
	{	this.childDocs.add(documentID);
	}

	public boolean postDocument(String filename, DocumentID documentID)
	{	int common = 0;
		int i;
		
		while (common < filename.length() &&
		       common < this.stem.length() &&
					  filename.charAt(common) == stem.charAt(common))
		{	common ++;
		}

		if (common == 0)
		{ return false;
		}

		if (common == this.stem.length())
		{	// post it in this node
			filename = filename.substring(this.stem.length());

			// select which sub-trie to attempt...
			for (i = 0; i < this.childTries.size(); i ++)
			{	FileTrie child = (FileTrie) this.childTries.get(i);
				
				if (child.postDocument(filename, documentID))
				{	break;
				}
			}

			if (i == this.childTries.size())
			{ FileTrie newChild = new FileTrie(filename);
				this.addChild(newChild);

				newChild.addDocument(documentID);
			}
		}
		else
		{	// divide this node at the 'common' point...
			FileTrie parentTrie = new FileTrie(this.stem.substring(0, common));
			this.parent.replaceChild(this, parentTrie);

			// truncate our root
			this.stem = this.stem.substring(common);

			// add ourselves to our new parent
			parentTrie.addChild(this);

			// trim back the file name
			filename = filename.substring(common);

			// recurse into posting the document
			this.postDocument(filename, documentID);
		}
		return true;
	}

	public boolean postFile(String filename)
	{	return this.postDocument(filename, null);
	}

	public List getDocuments(String filename)
	{	FileTrie leaf = this.getNode(filename);
		return leaf.childDocs;
	}

	public FileTrie getNode(String filename)
	{	int common = 0;
		int i;
		
		while (common < filename.length() &&
		       common < this.stem.length() &&
					 filename.charAt(common) == stem.charAt(common))
		{	common ++;
		}

		if (common == 0)
		{ return null;
		}

		if (common == this.stem.length())
		{	if (common == filename.length())
			{	return this;
			}

			filename = filename.substring(this.stem.length());

			// select which sub-trie to attempt...
			for (i = 0; i < this.childTries.size(); i ++)
			{	FileTrie child = (FileTrie) this.childTries.get(i);
				
				child = child.getNode(filename);

				if (child != null)
				{	return child;
				}
			}
		}
		return null;
	}
}
